
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: C_LFGList, UIParent, unpack, ZoneAbilityFrame

local BuffFrame = BuffFrame
local Mixin = Mixin

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Blizzard Miscellaneous")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	if not self:IsHooked(C_LFGList, "GetSearchResultInfo") then
		self:RawHook(C_LFGList, "GetSearchResultInfo", "C_LFGList_GetSearchResultInfo", true)
	end

	if not self:IsHooked(ZoneAbilityFrame, "UpdateDisplayedZoneAbilities") then
		self:SecureHook(ZoneAbilityFrame, "UpdateDisplayedZoneAbilities", "ZoneAbilityFrame_UpdateDisplayedZoneAbilities")
	end

	self:RegisterFrames()
	self:UpdateCollapseAndExpandButton()
	self:UpdateDefaultPanelPosition()
	self:CreateScrollBoxHooks()
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE")
end

function M:RegisterFrames()
	A:RegisterFrame("FloatingBattlePetTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingGarrisonFollowerAbilityTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingGarrisonFollowerTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingGarrisonMissionTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingGarrisonShipyardFollowerTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingPetBattleAbilityTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("ItemRefTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("StatusTrackingBarManager"):Hide()
	A:RegisterFrame("TalkingHeadFrame"):SetScale(0.8)
	A:RegisterFrame("ExtraAbilityContainer"):SetScale(0.8):AddCallback(function (_, frame) frame:SetClampedToScreen(false) end)
end

function M:UpdateCollapseAndExpandButton()
	local alpha = (self.config.showCollapseAndExpandButton or self.isClutterKeyDown) and 1 or 0

	BuffFrame.CollapseAndExpandButton:SetAlpha(alpha)
end

function M:UpdateDefaultPanelPosition()
	UIParent:SetAttribute("TOP_OFFSET", -250)
	UIParent:SetAttribute("LEFT_OFFSET", 20)
end

function M:IterateScrollBoxes()
	return coroutine.wrap(function ()
		for _, ScrollBox in next, {
			ProfessionsFrame.CraftingPage.CraftingOutputLog.ScrollBox,
			ProfessionsFrame.OrdersPage.OrderView.CraftingOutputLog.ScrollBox,
		} do
			coroutine.yield(ScrollBox)
		end
	end)
end

-- Disable ScrollToEnd until the Professions pane is opened to mitigate a bug where the TRADE_SKILL_ITEM_CRAFTED_RESULT event (which fires when applying a Fauna Rune to a weapon) could result in a Lua error.
function M:CreateScrollBoxHooks(event, addon)
	if not IsAddOnLoaded("Blizzard_Professions") then
		self:RegisterEvent("ADDON_LOADED", "CreateScrollBoxHooks")
	elseif event == "ADDON_LOADED" then
		if addon == "Blizzard_Professions" then
			self:UnregisterEvent("ADDON_LOADED")
			self:CreateScrollBoxHooks()
		end
	elseif not ProfessionsFrame then
		self:ScheduleTimer("CreateScrollBoxHooks", 1)
	else
		for ScrollBox in self:IterateScrollBoxes() do
			if not self:IsHooked(ScrollBox, "ScrollToEnd") then
				self:RawHook(ScrollBox, "ScrollToEnd", "ScrollBox_ScrollToEnd", true)
			end
		end

		if not self:IsHooked(ProfessionsFrame, "OnShow") then
			self:HookScript(ProfessionsFrame, "OnShow", "ProfessionsFrame_OnShow")
		end
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	self.isClutterKeyDown = state
	self:UpdateCollapseAndExpandButton()
end

function M:SIXUI_CONFIG_UPDATE(event, module, config, key, ...)
	if module == M:GetName() then
		if key == "showCollapseAndExpandButton" then
			self:UpdateCollapseAndExpandButton()
		end
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:C_LFGList_GetSearchResultInfo(...)
	local searchResultInfo = self.hooks[C_LFGList].GetSearchResultInfo(...)
	return searchResultInfo and Mixin(searchResultInfo, {playstyle = 0}) or nil
end

function M:ProfessionsFrame_OnShow()
	for ScrollBox in self:IterateScrollBoxes() do
		self:Unhook(ScrollBox, "ScrollToEnd")
	end
	
	self:Unhook(ProfessionsFrame, "OnShow")
end

function M:ScrollBox_ScrollToEnd()
	
end

-- Style the zone ability frame's spell buttons.
function M:ZoneAbilityFrame_UpdateDisplayedZoneAbilities()
	ZoneAbilityFrame:DisableDrawLayer("OVERLAY")

	for spellButton in ZoneAbilityFrame.SpellButtonContainer:EnumerateActive() do
		B:CreateBorder(spellButton, 2)
		spellButton.Icon:SetTexCoord(unpack(C.ZOOM_TEX_COORD))
		spellButton.NormalTexture:SetTexture(nil)
	end
end
